"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pitch = exports.raw = void 0;
const json5_1 = __importDefault(require("json5"));
const loader_utils_1 = __importDefault(require("loader-utils"));
const schema_utils_1 = __importDefault(require("schema-utils"));
const sharp_1 = __importDefault(require("sharp"));
const sizes_1 = require("./helpers/sizes");
const validation_1 = require("./helpers/validation");
const options_json_1 = __importDefault(require("./options.json"));
exports.raw = true;
function pitch(remainingRequest) {
    var _a;
    const callback = this.async();
    const options = loader_utils_1.default.getOptions(this);
    const queryObject = this.resourceQuery
        ? loader_utils_1.default.parseQuery(this.resourceQuery)
        : {};
    const fullOptions = {
        ...options,
        ...queryObject,
    };
    schema_utils_1.default(options_json_1.default, fullOptions, {
        name: "Image SrcSet Loader",
        baseDataPath: "options",
    });
    validation_1.validateSizes(fullOptions.sizes);
    const esModule = (_a = fullOptions.esModule) !== null && _a !== void 0 ? _a : true;
    generateSrcSetString(remainingRequest, this.loaders, this.loaderIndex, fullOptions, this.fs, this.resourcePath)
        .then((srcSetString) => {
        callback === null || callback === void 0 ? void 0 : callback(null, `${esModule ? "export default" : "module.exports ="}  \`${srcSetString}\`;`);
    })
        .catch((e) => {
        callback === null || callback === void 0 ? void 0 : callback(e);
    });
}
exports.pitch = pitch;
function default_1(source) {
    return source;
}
exports.default = default_1;
async function generateSrcSetString(remainingRequest, loaders, loaderIndex, options, fs, resourcePath) {
    let result = "";
    let width;
    const sizes = options.sizes;
    const maxDensity = sizes_1.getMaxDensity(sizes);
    const separator = ", ";
    const requireStart = '${require("-!';
    const requireEnd = '")}';
    for (const size of sizes) {
        if (!size) {
            result += `${requireStart}${addOptionsToResizeLoader(remainingRequest, loaders, loaderIndex, {}, options.customOptionsFactory)}${requireEnd}${separator}`;
            continue;
        }
        const resizeLoaderOption = sizes_1.getOptionFromSize(size, maxDensity);
        if (!options.scaleUp && resizeLoaderOption.width !== undefined) {
            if (width === undefined) {
                const buffer = fs.readFileSync(resourcePath);
                width = (await sharp_1.default(buffer).metadata()).width;
            }
            if (width !== undefined && resizeLoaderOption.width > width)
                continue;
        }
        result += `${requireStart}${addOptionsToResizeLoader(remainingRequest, loaders, loaderIndex, resizeLoaderOption, options.customOptionsFactory)}${requireEnd} ${size}${separator}`;
    }
    result = result.substring(0, result.length - 2);
    return result;
}
function addOptionsToResizeLoader(remainingRequest, loaders, loaderIndex, options, customOptionsFactory) {
    var _a;
    const nextLoader = loaders[loaderIndex + 1];
    const isNextLoaderQueryLoader = getIsLoaderQueryLoader(nextLoader);
    const resizeLoaderOptions = nextLoader.options;
    const resizeLoaderRequest = nextLoader.request;
    const resizeLoaderPath = nextLoader.path;
    if (customOptionsFactory)
        return remainingRequest.replace(resizeLoaderRequest, resizeLoaderPath +
            "?" +
            escapeJsonStringForLoader(json5_1.default.stringify(customOptionsFactory(options.width, options.scale, resizeLoaderOptions))));
    if (isNextLoaderQueryLoader) {
        const queryLoaderOptions = normalizeQueryLoaderOptions(resizeLoaderOptions);
        return remainingRequest.replace(resizeLoaderRequest, resizeLoaderPath +
            "?" +
            escapeJsonStringForLoader(json5_1.default.stringify({
                ...queryLoaderOptions,
                use: {
                    loader: queryLoaderOptions.use.loader,
                    options: {
                        scaleUp: true,
                        ...queryLoaderOptions.use.options,
                        ...options,
                        fileLoaderOptions: {
                            ...(_a = queryLoaderOptions.use.options) === null || _a === void 0 ? void 0 : _a.fileLoaderOptions,
                            esModule: false,
                        },
                    },
                },
            })));
    }
    return remainingRequest.replace(resizeLoaderRequest, resizeLoaderPath +
        "?" +
        escapeJsonStringForLoader(json5_1.default.stringify({
            scaleUp: true,
            ...resizeLoaderOptions,
            ...options,
            fileLoaderOptions: {
                ...resizeLoaderOptions === null || resizeLoaderOptions === void 0 ? void 0 : resizeLoaderOptions.fileLoaderOptions,
                esModule: false,
            },
        })));
}
function normalizeQueryLoaderOptions(options) {
    if (typeof options.use === "string")
        return {
            ...options,
            use: { loader: options.use },
        };
    return options;
}
function getIsLoaderQueryLoader(nextLoader) {
    if (nextLoader === undefined)
        throw "webpack-image-srcset-loader must be placed at the beginning and followed by a resize loader";
    if (nextLoader.path.includes("webpack-query-loader"))
        return true;
    return false;
}
function escapeJsonStringForLoader(s) {
    return s.replace(/\!/g, "\\\\x21");
}
